@@ -1,4 +1,4 @@
-;; $Id: bioperl-mode.el 16076 2009-09-14 19:00:48Z maj $
+;; $Id: bioperl-mode.el 16139 2009-09-18 21:11:23Z cjfields $
 
 ;; use multiple paths in bioperl-module-path
 
@@ -93,7 +93,7 @@
 (require 'bioperl-init)
 (require 'pod)
 
-(defconst bioperl-mode-revision "$Id: bioperl-mode.el 16076 2009-09-14 19:00:48Z maj $"
+(defconst bioperl-mode-revision "$Id: bioperl-mode.el 16139 2009-09-18 21:11:23Z cjfields $"
   "The revision string of bioperl-mode.el")
 
 ;;
@@ -143,6 +143,11 @@ The module filepath represented by the cached info is contained
 in `bioperl-cached-module'.")
 
 (defvar bioperl-cached-module nil
+  "Contains the module name whose method pod information is
+  currently stored in `bioperl-method-pod-cache'.
+The value is in double colon format.")
+
+(defvar bioperl-cached-pmfile nil
   "Contains the filepath whose method pod information is
   currently stored in `bioperl-method-pod-cache'.")
 
@@ -479,6 +484,7 @@ This function, when successful, also sets the cache vars
 	      (setq bioperl-method-pod-cache 
 		    (sort data (lambda (a b) (string-lessp (car a) (car b)))))
 	      (setq bioperl-cached-module module)
+	      (setq bioperl-cached-pmfile pmfile)
 	      ;; return the data alist for this module...
 	      bioperl-method-pod-cache )
 	  ;; the APPENDIX was not found...return nil
@@ -954,6 +960,7 @@ The module name for this method is assumed to be present in
 	    (insert ": " cur-content))
 	  (goto-char (point-min))
 	  (bioperl-view-mode)
+	  (set (make-local-variable 'bioperl-source-file) bioperl-cached-pmfile)
 	  (pop-to-buffer pod-buf)))
       )))
 
